local EventHandler = FlexBar2.OO.Mixin
{
	-- Custom event interface functions, act as advanced (and tweaked) versions of the AceEvent ones
	"RegisterEvent", -- Custom RegisterEvent function, Registers the event with AceEvent if it ain't already. It is table-based, read its code for details ;)
	"UnregisterEvent",
	"UnregisterAllEvents",

	"PostClick_UncheckButton", -- Uncheck the default checking of the button after it has been clicked
	"ACTIONBAR_SLOT_CHANGED_UpdateTextures", -- Update button icons when an actionbar slot changed
	"PLAYER_ENTERING_WORLD_LoadButtonTextures", -- Update the button icons after all information is available to the UI
	"SpellCast_Start", -- Triggered whenever a spellcast starts (whenever a button is clicked)
	"SpellCast_End", -- Triggered whenever a spellcast ends, be it on the SPELLCAST_FAILED or SPELLCAST_STOP event
	"Channeling_Start", -- Triggered when the player starts to channel a spell
	"Channeling_Stop", -- Triggered when the player stops channeling a spell
	"Buff_Gained", -- Triggered when the player gains a buff
	"Buff_Faded", -- Triggered when a buff fades from the player
}


function EventHandler:Activate()
	self.Frame.Button:SetScript("PostClick", self.PostClick_UncheckButton);
	self:RegisterEvent("ACTIONBAR_SLOT_CHANGED", "ACTIONBAR_SLOT_CHANGED_UpdateTextures");
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "PLAYER_ENTERING_WORLD_LoadButtonTextures");
	self:RegisterEvent("UNIT_SPELLCAST_START", "SpellCast_Start");
	self:RegisterEvent("UNIT_SPELLCAST_STOP", "SpellCast_End");
	self:RegisterEvent("UNIT_SPELLCAST_FAILED", "SpellCast_End");
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START","Channeling_Start");
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP", "Channeling_Stop");
end


function EventHandler:RegisterEvent(Event, MethodName)
	FlexBar2:Debug("Registering " , Event, " with " , MethodName);
	if(type(self[MethodName]) ~= "function") then
		return false
	end
	if(not FlexBar2:IsEventRegistered(Event)) then
		FlexBar2[Event] = function(...)
			for n ,v in pairs(FlexBar2.EventTable[Event]) do
				for _,v in pairs(v) do
					n[v](n, ...);
				end
			end
		end
		FlexBar2:RegisterEvent(Event);
	end
	if(type(FlexBar2.EventTable[Event]) ~= "table") then
		FlexBar2.EventTable[Event] = {};
	end
	if(type(FlexBar2.EventTable[Event][self]) ~= "table") then
		FlexBar2.EventTable[Event][self] = {};
	end
	table.insert(FlexBar2.EventTable[Event][self], MethodName);
end

function EventHandler:UnregisterEvent(Event, MethodName)
	if(MethodName) then
		return FlexBar2:RemoveByValue(FlexBar2.EventTable[Event][self], MethodName);
	else
		FlexBar2.EventTable[Event][self] = {};
	end
end


function EventHandler:PLAYER_ENTERING_WORLD_LoadButtonTextures(Addon)
	FlexBar2:Debug("PLAYER_ENTERING_WORLD_LoadButtonTextures");
	self:LoadCurrentTexture();
end

function EventHandler:PostClick_UncheckButton()
	FlexBar2:Debug("PostClick_UncheckButton");
	this:SetChecked(false);
end

function EventHandler:ACTIONBAR_SLOT_CHANGED_UpdateTextures(Addon, Slot)
	-- Update actionbar slot texture if this button's slot corresponds with the changed slot(s)
	if(type(self.Settings) == "table" and type(self.Settings.Attributes) == "table" and self.Settings.Attributes.type == "action" and self.Settings.Attributes.action == Slot) then
		FlexBar2:Debug("ACTIONBAR_SLOT_CHANGED_UpdateTextures");
		FlexBar2:TriggerEvent("ACTIONBAR_SLOT_CHANGED_UpdateTextures");
		self:LoadCurrentTexture();
	end
end

function EventHandler:SpellCast_Start(Addon, Unit)
	if(Unit == "player" and self.Settings and self.Settings.Attributes and self.Settings.Attributes.type == "action" and IsCurrentAction(self.Settings.Attributes.action)) then
		FlexBar2:Debug("SpellCast_Start"); -- Trigger the SpellCast_Start event and check the button
		FlexBar2:TriggerEvent("SpellCast_Start");
		self:SetChecked(true, "Casting");
	end
end

function EventHandler:SpellCast_End(Addon, Unit)
	if(Unit == "player" and self.Settings and self.Settings.Attributes and self.Settings.Attributes.type == "action") then
		FlexBar2:Debug("SpellCast_End"); -- We will take care of unchecking the button
		FlexBar2:TriggerEvent("SpellCast_End");
		self:SetChecked(false, "Casting");
	end
end

function EventHandler:Channeling_Start(Addon, Unit)
	if(Unit == "player" and self.Settings and self.Settings.Attributes and self.Settings.Attributes.type == "action" and IsCurrentAction(self.Settings.Attributes.action)) then
		FlexBar2:Debug("Channeling_Start"); -- Check the button for channeling
		FlexBar2:TriggerEvent("Channeling_Start");
		self:SetChecked(true, "Channeling");
	end
end

function EventHandler:Channeling_Stop(Addon, Unit)
	if(Unit == "player" and self.Settings and self.Settings.Attributes and self.Settings.Attributes.type == "action") then
		FlexBar2:Debug("Channeling_Stop"); -- We will take care of unchecking the button
		FlexBar2:TriggerEvent("Channeling_Stop");
		self.Frame.Button:SetChecked(false, "Channeling");
	end
end

function EventHandler:Buff_Gained(Addon, Slot)
	if(Unit == "player" and self.Settings and self.Settings.Attributes and self.Settings.Attributes.type == "action" self.Settings.Attributes.action == Slot) then
		FlexBar2:Debug("Buff_Gained");
		FlexBar2:TriggerEvent("Buff_Gained");
		
	end
end

function EventHandler:Buff_Faded(Addon, Slot)
	
end

FlexBar2:RegisterClass(EventHandler);
